package net.richardlord.smartcomponents.twitter.context
{
	import flash.events.Event;
	import flash.events.IEventDispatcher;
	
	import mx.core.IMXMLObject;
	import mx.events.FlexEvent;
	
	import net.richardlord.smartcomponents.twitter.data.TweetCollection;
	import net.richardlord.smartcomponents.twitter.data.User;
	import net.richardlord.smartcomponents.twitter.managers.*;
	import net.richardlord.smartcomponents.twitter.services.*;
	import net.richardlord.smartcomponents.twitter.services.parsers.*;
	import net.richardlord.smartcomponents.twitter.services.tools.TwitterRequest;
	
	import org.flaircode.oauth.OAuth;
	import org.swizframework.core.BeanProvider;
	import org.swizframework.core.IBeanFactory;
	import org.swizframework.core.IBeanProvider;
	import org.swizframework.core.ISwizConfig;
	import org.swizframework.core.Prototype;
	import org.swizframework.core.Swiz;
	import org.swizframework.core.SwizConfig;
	import org.swizframework.utils.DomainUtil;
	
	public class TwitterContext extends Swiz implements IMXMLObject
	{
		[Inspectable( defaultValue="", category="Common" )]
		public var oauthConsumerKey : String;
		
		[Inspectable( defaultValue="", category="Common" )]
		public var oauthConsumerSecret : String;
		
		private var oAuth:OAuth;

		public function TwitterContext()
		{
			super();
		}
		
		private function getConfig() : ISwizConfig
		{
			var newConfig:SwizConfig = new SwizConfig();
			newConfig.eventPackages = ["net.richardlord.smartcomponents.twitter.events"];
			newConfig.viewPackages = ["net.richardlord.smartcomponents.twitter"];
			return newConfig;
		}
		
		private function getBeanProvider() : IBeanProvider
		{
			var beans:Array = new Array();
			beans.push( new User() );
			beans.push( new TweetCollection() );
			beans.push( new AuthenticateManager() );
			beans.push(  oAuth );
			
			beans.push( new Prototype( HomeManager ) );
			beans.push( new Prototype( MoreManager ) );
			beans.push( new Prototype( RefreshManager ) );
			beans.push( new Prototype( SearchManager ) );
			beans.push( new Prototype( TweetManager ) );
			
			beans.push( new Prototype( HomeService ) );
			beans.push( new Prototype( MoreService ) );
			beans.push( new Prototype( RefreshService ) );
			beans.push( new Prototype( SearchService ) );
			beans.push( new Prototype( TweetService ) );

			beans.push( new Prototype( TwitterRequest ) );
			beans.push( new Prototype( SearchParser ) );
			beans.push( new Prototype( TimelineAtomParser ) );
			beans.push( new Prototype( TweetXmlParser ) );
			
			var beanProvider:BeanProvider = new BeanProvider( beans );
			return beanProvider;
		}
		
		public function initialized( document:Object, id:String ):void
		{
			if( document is IEventDispatcher && dispatcher == null )
			{
				dispatcher = IEventDispatcher( document );
				dispatcher.addEventListener( FlexEvent.PREINITIALIZE, handleContainerPreinitialize );
			}
			oAuth = new OAuth( oauthConsumerKey, oauthConsumerSecret );
			config = getConfig();
			beanProviders = [ getBeanProvider() ];
		}
		
		protected function handleContainerPreinitialize( event:Event ):void
		{
			dispatcher.removeEventListener( FlexEvent.PREINITIALIZE, handleContainerPreinitialize );
			domain = DomainUtil.getModuleDomain( dispatcher );
			init();
		}

	}
}