/*
 * Author: Richard Lord
 * Copyright (c) Richard Lord 2010
 * http://www.richardlord.net
 * 
 * Licence Agreement
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package net.richardlord.smartcomponents.twitter.data
{
	import flash.events.Event;
	import flash.events.EventDispatcher;
	
	import net.richardlord.smartcomponents.twitter.services.parsers.Parser;
	
	public class TweetCollection extends EventDispatcher
	{
		private var _tweets : Array;
		private var _id : uint;
		private var _title : String;
		private var _subtitle : String;
		private var _webUrl : String;
		private var _selfUrl : String;
		private var _refreshUrl : String;
		private var _nextUrl : String;
		private var _updated : Date;
		private var _itemsPerPage : uint;
		private var _language : String;
		private var _parser : Parser;
		
		public function TweetCollection()
		{
			_tweets = new Array();
		}
		
		[Bindable( event="tweetsChanged" )]
		public function get tweets() : Array
		{
			return _tweets;
		}
		
		public function set tweets( value : Array ) : void
		{
			if ( value != _tweets )
			{
				_tweets = value;
				dispatchEvent( new Event( "tweetsChanged" ) );
			}
		}
		
		[Bindable( event="idChanged" )]
		public function get id() : uint
		{
			return _id;
		}
		
		public function set id( value : uint ) : void
		{
			if ( value != _id )
			{
				_id = value;
				dispatchEvent( new Event( "idChanged" ) );
			}
		}
		
		[Bindable( event="titleChanged" )]
		public function get title() : String
		{
			return _title;
		}
		
		public function set title( value : String ) : void
		{
			if ( value != _title )
			{
				_title = value;
				dispatchEvent( new Event( "titleChanged" ) );
			}
		}
		
		[Bindable( event="subtitleChanged" )]
		public function get subtitle() : String
		{
			return _subtitle;
		}
		
		public function set subtitle( value : String ) : void
		{
			if ( value != _subtitle )
			{
				_subtitle = value;
				dispatchEvent( new Event( "subtitleChanged" ) );
			}
		}
		
		[Bindable( event="webUrlChanged" )]
		public function get webUrl() : String
		{
			return _webUrl;
		}
		
		public function set webUrl( value : String ) : void
		{
			if ( value != _webUrl )
			{
				_webUrl = value;
				dispatchEvent( new Event( "webUrlChanged" ) );
			}
		}
		
		[Bindable( event="selfUrlChanged" )]
		public function get selfUrl() : String
		{
			return _selfUrl;
		}
		
		public function set selfUrl( value : String ) : void
		{
			if ( value != _selfUrl )
			{
				_selfUrl = value;
				dispatchEvent( new Event( "selfUrlChanged" ) );
			}
		}
		
		[Bindable( event="refreshUrlChanged" )]
		public function get refreshUrl() : String
		{
			return _refreshUrl;
		}
		
		public function set refreshUrl( value : String ) : void
		{
			if ( value != _refreshUrl )
			{
				_refreshUrl = value;
				dispatchEvent( new Event( "refreshUrlChanged" ) );
			}
		}
		
		[Bindable( event="nextUrlChanged" )]
		public function get nextUrl() : String
		{
			return _nextUrl;
		}
		
		public function set nextUrl( value : String ) : void
		{
			if ( value != _nextUrl )
			{
				_nextUrl = value;
				dispatchEvent( new Event( "nextUrlChanged" ) );
			}
		}
		
		[Bindable( event="updatedChanged" )]
		public function get updated() : Date
		{
			return _updated;
		}
		
		public function set updated( value : Date ) : void
		{
			if ( value != _updated )
			{
				_updated = value;
				dispatchEvent( new Event( "updatedChanged" ) );
			}
		}
		
		[Bindable( event="itemsPerPageChanged" )]
		public function get itemsPerPage() : uint
		{
			return _itemsPerPage;
		}
		
		public function set itemsPerPage( value : uint ) : void
		{
			if ( value != _itemsPerPage )
			{
				_itemsPerPage = value;
				dispatchEvent( new Event( "itemsPerPageChanged" ) );
			}
		}
		
		[Bindable( event="languageChanged" )]
		public function get language() : String
		{
			return _language;
		}
		
		public function set language( value : String ) : void
		{
			if ( value != _language )
			{
				_language = value;
				dispatchEvent( new Event( "languageChanged" ) );
			}
		}
		
		[Bindable( event="parserChanged" )]
		public function get parser() : Parser
		{
			return _parser;
		}
		
		public function set parser( value : Parser ) : void
		{
			if ( value != _parser )
			{
				_parser = value;
				dispatchEvent( new Event( "parserChanged" ) );
			}
		}
		
		public function assign( other : TweetCollection ) : void
		{
			tweets = other.tweets;
			id = other.id;
			title = other.title;
			webUrl = other.webUrl;
			selfUrl = other.selfUrl;
			refreshUrl = other.refreshUrl;
			nextUrl = other.nextUrl;
			updated = other.updated;
			itemsPerPage = other.itemsPerPage;
			language = other.language;
			parser = other.parser;
		}
		
		public function prependTweet( tweet : Tweet ) : void
		{
			for each ( var other : Tweet in tweets )
			{
				if ( other.id == tweet.id )
				{
					return;
				}
			}
			tweets.unshift( tweet );
			dispatchEvent( new Event( "tweetsChanged" ) );
		}
		
		public function append( other : TweetCollection ) : void
		{
			var len : int = other.tweets.length;
			for ( var i : int = 0; i < len; ++i )
			{
				var tweet : Tweet = other.tweets[ i ];
				for ( var j : int = tweets.length - 1; j >= 0; --j )
				{
					if ( Tweet( tweets[ j ] ).id == tweet.id )
					{
						tweets.splice( j, 1 );
						continue;
					}
				}
				break;
			}
			tweets = tweets.concat( other.tweets );
			if ( other.nextUrl )
			{
				nextUrl = other.nextUrl;
			}
		}
		
		public function prepend( other : TweetCollection ) : void
		{
			for ( var i : int = other.tweets.length - 1; i >= 0; --i )
			{
				var tweet : Tweet = other.tweets[ i ];
				var len : int = tweets.length;
				for ( var j : int = 0; j < len; ++j )
				{
					if ( Tweet( tweets[ j ] ).id == tweet.id )
					{
						tweets.splice( j, 1 );
						--len;
						continue;
					}
				}
				break;
			}
			tweets = other.tweets.concat( tweets );
			if ( other.refreshUrl )
			{
				refreshUrl = other.refreshUrl;
			}
		}
	}
}